/**
 * @encoding     UTF-8
 * @package      WordPress Demo Content Manager
 * @link         https://codecanyon.net/item/wordpress-demo-content-manager/20669604
 * @copyright    Copyright (C) 2018 Torbara (http://torbara.com). All rights reserved.
 * @license      Envato Standard License http://themeforest.net/licenses/standard?ref=torbara
 * @author       Vladislav Dolgolenko <vladislavdolgolenko.com>
 * @support      <help@vladislavdolgolenko.com>
 */

(function($, Backbone, dm){
    
    "use strict";
    
    dm.ModelStatus = Backbone.Model.extend({
        
        initialize: function () {
            this.on('change', this.processController, this);
            this.startUpdating();
        },
        
        logQuery: function () {
            var self = this;
            $.ajax(this.get('log_url'), {
                'cache': false,
                'success' : function(data) {
                    self.set('log', data);
                }
            });
        },
        
        helpQuery: function () {
            var self = this;
            $.ajax(this.get('ajax_url'), {
                'cache': false,
                'data' : {action: 'dm_processing_help'},
                'method' : 'POST',
                'success' : function(json) {
                    var data = JSON.parse(json);
                    self.set('help_html', data['html']);
                }
            });
        },
        
        dataQuery: function () {
            var self = this;
            $.ajax(this.get('ajax_url'), {
                'cache': false,
                'data' : {action: 'dm_data_action'},
                'method' : 'POST',
                'success' : function(json) {
                    if (!json) {
                        return;
                    }
                    
                    var data = JSON.parse(json);
                    if (data === undefined || !data) {
                        return;
                    }
                    
                    var status = data.current_status !== undefined ? data.current_status : null;
                    var demo_id = data.processing_demo ? data.processing_demo : null;
                    var counts = data.processing_steps_count !== undefined ? data.processing_steps_count : null;
                    var pid = data.pid !== undefined ? data.pid : null;
                    var process_id = data.process_id ? data.process_id : null;
                    var process_name = data.process_name ? data.process_name : null;
                    self.set('current_progress', status);
                    self.set('processing_demo', demo_id);
                    self.set('process_id', process_id);
                    self.set('process_name', process_name);
                    self.set('counts', counts);
                    self.set('pid', pid);
                }
            });
        },
        
        processController: function () {
            if (this.get('pid')) {
                this.trigger('processing-start');
            } else if (this.get('current_progress') !== null) {
                this.trigger('processing-error');
                this.helpQuery();
            } else {
                this.trigger('processing-success');
            }
        },
        
        stopUpdating: function () {
            if (this.interval !== undefined) {
                clearInterval(this.interval);
                this.interval = null;
            }
        },
        
        startUpdating: function () {
            var self = this;
            
            if (this.interval) {
                return;
            }
            
            this.interval = setInterval(function(){
                self.logQuery();
                self.dataQuery();
            }, 1000);
        },
        
        processing: function () {
            var confirm_proccess = confirm('Do you really want to perform this action?');
            if (confirm_proccess !== true) {
                return;
            }
            
            var demo_id = this.get('processing_demo');
            var url = this.get('ajax_url');
            var process_id = this.get('process_id');
            var self = this;
            $.ajax(url, {
                'method' : 'POST',
                'data' : {demo_id: demo_id, action: 'dm_processing', process: process_id},
                'success': function(){
                    self.trigger('processing-query-success');
                }
            });
            
            this.dataQuery();
        },
        
        aborting: function () {
            var confirm_proccess = confirm('Do you really want to perform this action?');
            if (confirm_proccess !== true) {
                return;
            }
            
            var demo_id = this.get('processing_demo');
            var url = this.get('ajax_url');
            var process_id = this.get('process_id');
            var self = this;
            $.ajax(url, {
                'method' : 'POST',
                'data' : {demo_id: demo_id, action: 'dm_aborting', process: process_id},
                'success': function(){
                    self.trigger('processing-query-success');
                }
            });
            
            this.dataQuery();
        },
        
        kill: function () {
            var confirm_proccess = confirm('This action do not stop current process, use this if processing really working more than 10 minutes without results. Do you really want to perform this action?');
            if (confirm_proccess !== true) {
                return;
            }
            
            var url = this.get('ajax_url');
            var self = this;
            $.ajax(url, {
                'method' : 'POST',
                'data' : {action: 'dm_kill'},
                'success': function(){
                    self.trigger('processing-query-success');
                }
            });
            
            this.dataQuery();
        },
        
        // Processing methods
        
        doInstallation: function () {
            this.set('process_id', 'dm_Installer');
            this.processing();
        },
        
        doUninstallation: function () {
            this.set('process_id', 'dm_Installer');
            this.aborting();
        },
        
        doActivation: function () {
            this.set('process_id', 'dm_activator');
            this.processing();
        },
        
        doDeactivation: function () {
            this.set('process_id', 'dm_activator');
            this.aborting();
        },
        
        doPaking: function () {
            this.set('process_id', 'dm_Creator');
            this.processing();
        },
        
        doUnPaking: function () {
            this.set('process_id', 'dm_Creator');
            this.aborting();
        },
        
        doUplaod: function () {
            this.set('process_id', 'upload_demo');
            this.processing();
        },
        
        doDelete: function () {
            this.set('process_id', 'upload_demo');
            this.aborting();
        },
        
        doDeploying: function () {
            this.set('process_id', 'deploying_demo');
            this.processing();
        },
        
        doRemouteFromRepository: function () {
            this.set('process_id', 'deploying_demo');
            this.aborting();
        },
        
        doMakeZip: function () {
            this.set('process_id', 'make_zip');
            this.processing();
        }
        
    });
    
})(jQuery, Backbone, window.wp_demo_manager);

